# In consumers.py
from channels import Group, Channel
from .models import ChatMessage
from otree.models import Participant
import json

def get_chat_group(channel):
    return 'otreechat-{}'.format(channel)

from channels.generic.websockets import JsonWebsocketConsumer

def msg_consumer(message):
    content = message.content

    # For now, don't create a model because we
    # don't yet have a way to export this table.
    # currently oTree admin is not extensible.

    channel = content['channel']
    grp = get_chat_group(channel)

    # list containing 1 element
    Group(grp).send({'text': json.dumps([content])})

    ChatMessage.objects.create(
        participant=Participant.objects.get(id=content['participant_id']),
        channel=content['channel'],
        body=content['body'],
        nickname=content['nickname']
    )


class ChatConsumer(JsonWebsocketConsumer):

    # Set to True if you want it, else leave it out
    strict_ordering = False

    def connection_groups(self, **kwargs):
        """
        Called to return the list of groups to automatically add/remove
        this connection to/from.
        """
        return [get_chat_group(kwargs['channel'])]

    def connect(self, message, **kwargs):
        history = ChatMessage.objects.filter(
            channel=kwargs['channel']).order_by('timestamp').values(
                'channel', 'nickname', 'body', 'participant_id'
        )

        # Convert ValuesQuerySet to list
        self.send(list(history))

    def receive(self, content, **kwargs):
        # Stick the message onto the processing queue
        Channel("otree.chat_messages").send(content)



from django import template
register = template.Library()
import re

@register.inclusion_tag('otreechat/widget.html', takes_context=True)
def chat(context, *args, **kwargs):
    player = context['player']
    group = context['group']
    Constants = context['Constants']

    kwargs.setdefault('channel', group.id)
    kwargs.setdefault('nickname', 'Player {}'.format(player.id_in_group))

    # prefix the channel name with session code and app name
    context['channel'] = '{}-{}-{}'.format(
        context['session'].code,
        Constants.name_in_url,
        kwargs['channel']
    )

    context['nickname'] = kwargs['nickname']

    # channel name should not contain illegal chars,
    # so that it can be used in JS and URLs
    if not re.match(r'^[a-zA-Z0-9_-]+$', context['channel']):
        raise ValueError(
            "'channel' can only contain ASCII letters, numbers, underscores, and hyphens. "
            "Value given was: {}".format(context['channel']))

    return context